@@ -20,7 +20,7 @@
 
 #include <config.h>
 
-#if !((defined _WIN32 || defined __WIN32__) && !defined __CYGWIN__)
+#if !(defined _WIN32 && !defined __CYGWIN__)
 /* Unix API.  */
 
 /* Specification.  */
