@@ -573,20 +573,6 @@ virDomainPCIAddressSetGrow(virDomainPCIAddressSetPtr addrs,
 }
 
 
-char *
-virDomainPCIAddressAsString(virPCIDeviceAddressPtr addr)
-{
-    char *str;
-
-    ignore_value(virAsprintf(&str, "%.4x:%.2x:%.2x.%.1x",
-                             addr->domain,
-                             addr->bus,
-                             addr->slot,
-                             addr->function));
-    return str;
-}
-
-
 /*
  * Check if the PCI slot is used by another device.
  */
@@ -618,7 +604,7 @@ virDomainPCIAddressReserveAddrInternal(virDomainPCIAddressSetPtr addrs,
     virErrorNumber errType = (fromConfig
                               ? VIR_ERR_XML_ERROR : VIR_ERR_INTERNAL_ERROR);
 
-    if (!(addrStr = virDomainPCIAddressAsString(addr)))
+    if (!(addrStr = virPCIDeviceAddressAsString(addr)))
         goto cleanup;
 
     /* Add an extra bus if necessary */
@@ -703,7 +689,7 @@ virDomainPCIAddressEnsureAddr(virDomainPCIAddressSetPtr addrs,
     if (!flags)
        return 0;
 
-    if (!(addrStr = virDomainPCIAddressAsString(&dev->addr.pci)))
+    if (!(addrStr = virPCIDeviceAddressAsString(&dev->addr.pci)))
         goto cleanup;
 
     if (virDeviceInfoPCIAddressIsPresent(dev)) {
@@ -784,7 +770,7 @@ virDomainPCIAddressFindUnusedFunctionOnBus(virDomainPCIAddressBusPtr bus,
 
     *found = false;
 
-    if (!(addrStr = virDomainPCIAddressAsString(searchAddr)))
+    if (!(addrStr = virPCIDeviceAddressAsString(searchAddr)))
         goto cleanup;
 
     if (!virDomainPCIAddressFlagsCompatible(searchAddr, addrStr, bus->flags,
