@@ -23,14 +23,10 @@
 
 #include <config.h>
 
-#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <errno.h>
-#include <stdio.h>
 #include <fcntl.h>
-#include <string.h>
 
 #include "virerror.h"
 #include "datatypes.h"
