@@ -27,6 +27,7 @@
 # include "virerror.h"
 # include "datatypes.h"
 # include "virarch.h"
+# include "domain_capabilities.h"
 # include "cpu_conf.h"
 # include "cpu_x86_data.h"
 # include "cpu_ppc64_data.h"
@@ -52,9 +53,7 @@ typedef virCPUCompareResult
 typedef int
 (*cpuArchDecode)    (virCPUDefPtr cpu,
                      const virCPUData *data,
-                     const char **models,
-                     unsigned int nmodels,
-                     const char *preferred);
+                     virDomainCapsCPUModelsPtr models);
 
 typedef int
 (*cpuArchEncode)    (virArch arch,
@@ -71,15 +70,14 @@ typedef void
 
 typedef int
 (*virCPUArchGetHost)(virCPUDefPtr cpu,
-                     const char **models,
-                     unsigned int nmodels);
+                     virDomainCapsCPUModelsPtr models);
 
 typedef virCPUDefPtr
-(*cpuArchBaseline)  (virCPUDefPtr *cpus,
-                     unsigned int ncpus,
-                     const char **models,
-                     unsigned int nmodels,
-                     bool migratable);
+(*virCPUArchBaseline)(virCPUDefPtr *cpus,
+                      unsigned int ncpus,
+                      virDomainCapsCPUModelsPtr models,
+                      const char **features,
+                      bool migratable);
 
 typedef int
 (*virCPUArchUpdate)(virCPUDefPtr guest,
@@ -109,8 +107,7 @@ typedef int
 
 typedef int
 (*virCPUArchTranslate)(virCPUDefPtr cpu,
-                       const char **models,
-                       unsigned int nmodels);
+                       virDomainCapsCPUModelsPtr models);
 
 typedef int
 (*virCPUArchConvertLegacy)(virCPUDefPtr cpu);
@@ -133,7 +130,7 @@ struct cpuArchDriver {
     cpuArchEncode       encode;
     cpuArchDataFree     dataFree;
     virCPUArchGetHost   getHost;
-    cpuArchBaseline     baseline;
+    virCPUArchBaseline baseline;
     virCPUArchUpdate    update;
     virCPUArchUpdateLive updateLive;
     virCPUArchCheckFeature checkFeature;
@@ -165,9 +162,7 @@ virCPUCompare(virArch arch,
 int
 cpuDecode   (virCPUDefPtr cpu,
              const virCPUData *data,
-             const char **models,
-             unsigned int nmodels,
-             const char *preferred)
+             virDomainCapsCPUModelsPtr models)
     ATTRIBUTE_NONNULL(1) ATTRIBUTE_NONNULL(2);
 
 int
@@ -194,18 +189,18 @@ virCPUDefPtr
 virCPUGetHost(virArch arch,
               virCPUType type,
               virNodeInfoPtr nodeInfo,
-              const char **models,
-              unsigned int nmodels);
+              virDomainCapsCPUModelsPtr models);
 
 virCPUDefPtr
 virCPUProbeHost(virArch arch);
 
 virCPUDefPtr
-cpuBaseline (virCPUDefPtr *cpus,
-             unsigned int ncpus,
-             const char **models,
-             unsigned int nmodels,
-             bool migratable);
+virCPUBaseline(virArch arch,
+               virCPUDefPtr *cpus,
+               unsigned int ncpus,
+               virDomainCapsCPUModelsPtr models,
+               const char **features,
+               bool migratable);
 
 int
 virCPUUpdate(virArch arch,
@@ -235,8 +230,7 @@ virCPUDataCheckFeature(const virCPUData *data,
 
 bool
 virCPUModelIsAllowed(const char *model,
-                     const char **models,
-                     unsigned int nmodels)
+                     virDomainCapsCPUModelsPtr models)
     ATTRIBUTE_NONNULL(1);
 
 int
@@ -245,8 +239,7 @@ virCPUGetModels(virArch arch, char ***models);
 int
 virCPUTranslate(virArch arch,
                 virCPUDefPtr cpu,
-                const char **models,
-                unsigned int nmodels)
+                virDomainCapsCPUModelsPtr models)
     ATTRIBUTE_NONNULL(2);
 
 int
