@@ -56,15 +56,26 @@ struct _virLockManagerLockDaemonResource {
 };
 
 struct _virLockManagerLockDaemonPrivate {
-    unsigned char uuid[VIR_UUID_BUFLEN];
-    char *name;
-    int id;
-    pid_t pid;
+    virLockManagerObjectType type;
+    union {
+        struct {
+            unsigned char uuid[VIR_UUID_BUFLEN];
+            char *name;
+            int id;
+            pid_t pid;
+
+            bool hasRWDisks;
+        } dom;
+
+        struct {
+            unsigned char uuid[VIR_UUID_BUFLEN];
+            char *name;
+            pid_t pid;
+        } daemon;
+    } t;
 
     size_t nresources;
     virLockManagerLockDaemonResourcePtr resources;
-
-    bool hasRWDisks;
 };
 
 
@@ -156,10 +167,30 @@ virLockManagerLockDaemonConnectionRegister(virLockManagerPtr lock,
     memset(&args, 0, sizeof(args));
 
     args.flags = 0;
-    memcpy(args.owner.uuid, priv->uuid, VIR_UUID_BUFLEN);
-    args.owner.name = priv->name;
-    args.owner.id = priv->id;
-    args.owner.pid = priv->pid;
+
+    switch (priv->type) {
+    case VIR_LOCK_MANAGER_OBJECT_TYPE_DOMAIN:
+        memcpy(args.owner.uuid, priv->t.dom.uuid, VIR_UUID_BUFLEN);
+        args.owner.name = priv->t.dom.name;
+        args.owner.id = priv->t.dom.id;
+        args.owner.pid = priv->t.dom.pid;
+        break;
+
+    case VIR_LOCK_MANAGER_OBJECT_TYPE_DAEMON:
+        memcpy(args.owner.uuid, priv->t.daemon.uuid, VIR_UUID_BUFLEN);
+        args.owner.name = priv->t.daemon.name;
+        args.owner.pid = priv->t.daemon.pid;
+        /* This one should not be needed. However, virtlockd
+         * checks for ID because not every domain has a PID. */
+        args.owner.id = priv->t.daemon.pid;
+        break;
+
+    default:
+        virReportError(VIR_ERR_INTERNAL_ERROR,
+                       _("Unknown lock manager object type %d"),
+                       priv->type);
+        return -1;
+    }
 
     if (virNetClientProgramCall(program,
                                 client,
@@ -340,8 +371,10 @@ static int virLockManagerLockDaemonInit(unsigned int version,
     driver->requireLeaseForDisks = true;
     driver->autoDiskLease = true;
 
-    if (virLockManagerLockDaemonLoadConfig(configFile) < 0)
+    if (configFile &&
+        virLockManagerLockDaemonLoadConfig(configFile) < 0) {
         goto error;
+    }
 
     if (driver->autoDiskLease) {
         if (driver->fileLockSpaceDir &&
@@ -369,33 +402,52 @@ static int virLockManagerLockDaemonDeinit(void)
     if (!driver)
         return 0;
 
+    VIR_FREE(driver->scsiLockSpaceDir);
+    VIR_FREE(driver->lvmLockSpaceDir);
     VIR_FREE(driver->fileLockSpaceDir);
     VIR_FREE(driver);
 
     return 0;
 }
 
-static void virLockManagerLockDaemonFree(virLockManagerPtr lock)
+static void
+virLockManagerLockDaemonPrivateFree(virLockManagerLockDaemonPrivatePtr priv)
 {
-    virLockManagerLockDaemonPrivatePtr priv = lock->privateData;
     size_t i;
 
     if (!priv)
         return;
 
-    lock->privateData = NULL;
-
     for (i = 0; i < priv->nresources; i++) {
         VIR_FREE(priv->resources[i].lockspace);
         VIR_FREE(priv->resources[i].name);
     }
     VIR_FREE(priv->resources);
 
-    VIR_FREE(priv->name);
+    switch (priv->type) {
+    case VIR_LOCK_MANAGER_OBJECT_TYPE_DOMAIN:
+        VIR_FREE(priv->t.dom.name);
+        break;
 
+    case VIR_LOCK_MANAGER_OBJECT_TYPE_DAEMON:
+        VIR_FREE(priv->t.daemon.name);
+        break;
+
+    default:
+        break;
+    }
     VIR_FREE(priv);
 }
 
+static void virLockManagerLockDaemonFree(virLockManagerPtr lock)
+{
+    if (!lock)
+        return;
+
+    virLockManagerLockDaemonPrivateFree(lock->privateData);
+    lock->privateData = NULL;
+}
+
 
 static int virLockManagerLockDaemonNew(virLockManagerPtr lock,
                                        unsigned int type,
@@ -403,51 +455,88 @@ static int virLockManagerLockDaemonNew(virLockManagerPtr lock,
                                        virLockManagerParamPtr params,
                                        unsigned int flags)
 {
-    virLockManagerLockDaemonPrivatePtr priv;
+    virLockManagerLockDaemonPrivatePtr priv = NULL;
     size_t i;
+    int ret = -1;
 
     virCheckFlags(VIR_LOCK_MANAGER_NEW_STARTED, -1);
 
     if (VIR_ALLOC(priv) < 0)
         return -1;
-    lock->privateData = priv;
 
-    switch (type) {
+    priv->type = type;
+
+    switch ((virLockManagerObjectType) type) {
     case VIR_LOCK_MANAGER_OBJECT_TYPE_DOMAIN:
         for (i = 0; i < nparams; i++) {
             if (STREQ(params[i].key, "uuid")) {
-                memcpy(priv->uuid, params[i].value.uuid, VIR_UUID_BUFLEN);
+                memcpy(priv->t.dom.uuid, params[i].value.uuid, VIR_UUID_BUFLEN);
             } else if (STREQ(params[i].key, "name")) {
-                if (VIR_STRDUP(priv->name, params[i].value.str) < 0)
-                    return -1;
+                if (VIR_STRDUP(priv->t.dom.name, params[i].value.str) < 0)
+                    goto cleanup;
             } else if (STREQ(params[i].key, "id")) {
-                priv->id = params[i].value.iv;
+                priv->t.dom.id = params[i].value.iv;
             } else if (STREQ(params[i].key, "pid")) {
-                priv->pid = params[i].value.iv;
+                priv->t.dom.pid = params[i].value.iv;
             } else if (STREQ(params[i].key, "uri")) {
                 /* ignored */
             } else {
                 virReportError(VIR_ERR_INTERNAL_ERROR,
-                               _("Unexpected parameter %s for object"),
+                               _("Unexpected parameter %s for domain object"),
                                params[i].key);
+                goto cleanup;
             }
         }
-        if (priv->id == 0) {
+        if (priv->t.dom.id == 0) {
             virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                            _("Missing ID parameter for domain object"));
-            return -1;
+            goto cleanup;
         }
-        if (priv->pid == 0)
+        if (priv->t.dom.pid == 0)
             VIR_DEBUG("Missing PID parameter for domain object");
-        if (!priv->name) {
+        if (!priv->t.dom.name) {
             virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                            _("Missing name parameter for domain object"));
-            return -1;
+            goto cleanup;
         }
-        if (!virUUIDIsValid(priv->uuid)) {
+        if (!virUUIDIsValid(priv->t.dom.uuid)) {
             virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
                            _("Missing UUID parameter for domain object"));
-            return -1;
+            goto cleanup;
+        }
+        break;
+
+    case VIR_LOCK_MANAGER_OBJECT_TYPE_DAEMON:
+        for (i = 0; i < nparams; i++) {
+            if (STREQ(params[i].key, "uuid")) {
+                memcpy(priv->t.daemon.uuid, params[i].value.uuid, VIR_UUID_BUFLEN);
+            } else if (STREQ(params[i].key, "name")) {
+                if (VIR_STRDUP(priv->t.daemon.name, params[i].value.str) < 0)
+                    goto cleanup;
+            } else if (STREQ(params[i].key, "pid")) {
+                priv->t.daemon.pid = params[i].value.iv;
+            } else {
+                virReportError(VIR_ERR_INTERNAL_ERROR,
+                               _("Unexpected parameter %s for daemon object"),
+                               params[i].key);
+                goto cleanup;
+            }
+        }
+
+        if (!virUUIDIsValid(priv->t.daemon.uuid)) {
+            virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                           _("Missing UUID parameter for daemon object"));
+            goto cleanup;
+        }
+        if (!priv->t.daemon.name) {
+            virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                           _("Missing name parameter for daemon object"));
+            goto cleanup;
+        }
+        if (priv->t.daemon.pid == 0) {
+            virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                           _("Missing PID parameter for daemon object"));
+            goto cleanup;
         }
         break;
 
@@ -455,10 +544,14 @@ static int virLockManagerLockDaemonNew(virLockManagerPtr lock,
         virReportError(VIR_ERR_INTERNAL_ERROR,
                        _("Unknown lock manager object type %d"),
                        type);
-        return -1;
+        goto cleanup;
     }
 
-    return 0;
+    VIR_STEAL_PTR(lock->privateData, priv);
+    ret = 0;
+ cleanup:
+    virLockManagerLockDaemonPrivateFree(priv);
+    return ret;
 }
 
 
@@ -472,7 +565,8 @@ static int virLockManagerLockDaemonAddResource(virLockManagerPtr lock,
     virLockManagerLockDaemonPrivatePtr priv = lock->privateData;
     char *newName = NULL;
     char *newLockspace = NULL;
-    bool autoCreate = false;
+    int newFlags = 0;
+    int ret = -1;
 
     virCheckFlags(VIR_LOCK_MANAGER_RESOURCE_READONLY |
                   VIR_LOCK_MANAGER_RESOURCE_SHARED, -1);
@@ -480,134 +574,194 @@ static int virLockManagerLockDaemonAddResource(virLockManagerPtr lock,
     if (flags & VIR_LOCK_MANAGER_RESOURCE_READONLY)
         return 0;
 
-    switch (type) {
-    case VIR_LOCK_MANAGER_RESOURCE_TYPE_DISK:
-        if (params || nparams) {
-            virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
-                           _("Unexpected parameters for disk resource"));
-            return -1;
-        }
-        if (!driver->autoDiskLease) {
-            if (!(flags & (VIR_LOCK_MANAGER_RESOURCE_SHARED |
-                           VIR_LOCK_MANAGER_RESOURCE_READONLY)))
-                priv->hasRWDisks = true;
-            return 0;
-        }
+    switch (priv->type) {
+    case VIR_LOCK_MANAGER_OBJECT_TYPE_DOMAIN:
 
-        /* XXX we should somehow pass in TYPE=BLOCK info
-         * from the domain_lock code, instead of assuming /dev
-         */
-        if (STRPREFIX(name, "/dev") &&
-            driver->lvmLockSpaceDir) {
-            VIR_DEBUG("Trying to find an LVM UUID for %s", name);
-            if (virStorageFileGetLVMKey(name, &newName) < 0)
-                goto error;
-
-            if (newName) {
-                VIR_DEBUG("Got an LVM UUID %s for %s", newName, name);
-                if (VIR_STRDUP(newLockspace, driver->lvmLockSpaceDir) < 0)
-                    goto error;
-                autoCreate = true;
-                break;
+        switch ((virLockManagerResourceType) type) {
+        case VIR_LOCK_MANAGER_RESOURCE_TYPE_DISK:
+            if (params || nparams) {
+                virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                               _("Unexpected parameters for disk resource"));
+                goto cleanup;
             }
-            virResetLastError();
-            /* Fallback to generic non-block code */
-        }
-
-        if (STRPREFIX(name, "/dev") &&
-            driver->scsiLockSpaceDir) {
-            VIR_DEBUG("Trying to find an SCSI ID for %s", name);
-            if (virStorageFileGetSCSIKey(name, &newName) < 0)
-                goto error;
-
-            if (newName) {
-                VIR_DEBUG("Got an SCSI ID %s for %s", newName, name);
-                if (VIR_STRDUP(newLockspace, driver->scsiLockSpaceDir) < 0)
-                    goto error;
-                autoCreate = true;
-                break;
+            if (!driver->autoDiskLease) {
+                if (!(flags & (VIR_LOCK_MANAGER_RESOURCE_SHARED |
+                               VIR_LOCK_MANAGER_RESOURCE_READONLY)))
+                    priv->t.dom.hasRWDisks = true;
+                return 0;
             }
-            virResetLastError();
-            /* Fallback to generic non-block code */
-        }
 
-        if (driver->fileLockSpaceDir) {
-            if (VIR_STRDUP(newLockspace, driver->fileLockSpaceDir) < 0)
-                goto error;
-            if (virCryptoHashString(VIR_CRYPTO_HASH_SHA256, name, &newName) < 0)
-                goto error;
-            autoCreate = true;
-            VIR_DEBUG("Using indirect lease %s for %s", newName, name);
-        } else {
-            if (VIR_STRDUP(newLockspace, "") < 0)
-                goto error;
-            if (VIR_STRDUP(newName, name) < 0)
-                goto error;
-            VIR_DEBUG("Using direct lease for %s", name);
-        }
+            /* XXX we should somehow pass in TYPE=BLOCK info
+             * from the domain_lock code, instead of assuming /dev
+             */
+            if (STRPREFIX(name, "/dev") &&
+                driver->lvmLockSpaceDir) {
+                VIR_DEBUG("Trying to find an LVM UUID for %s", name);
+                if (virStorageFileGetLVMKey(name, &newName) < 0)
+                    goto cleanup;
 
-        break;
-    case VIR_LOCK_MANAGER_RESOURCE_TYPE_LEASE: {
-        size_t i;
-        char *path = NULL;
-        char *lockspace = NULL;
-        for (i = 0; i < nparams; i++) {
-            if (STREQ(params[i].key, "offset")) {
-                if (params[i].value.ul != 0) {
-                    virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
-                                   _("Offset must be zero for this lock manager"));
-                    return -1;
+                if (newName) {
+                    VIR_DEBUG("Got an LVM UUID %s for %s", newName, name);
+                    if (VIR_STRDUP(newLockspace, driver->lvmLockSpaceDir) < 0)
+                        goto cleanup;
+                    newFlags |= VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_AUTOCREATE;
+                    break;
                 }
-            } else if (STREQ(params[i].key, "lockspace")) {
-                lockspace = params[i].value.str;
-            } else if (STREQ(params[i].key, "path")) {
-                path = params[i].value.str;
-            } else {
-                virReportError(VIR_ERR_INTERNAL_ERROR,
-                               _("Unexpected parameter %s for lease resource"),
-                               params[i].key);
-                return -1;
+                virResetLastError();
+                /* Fallback to generic non-block code */
             }
-        }
-        if (!path || !lockspace) {
-            virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
-                           _("Missing path or lockspace for lease resource"));
-            return -1;
-        }
-        if (virAsprintf(&newLockspace, "%s/%s",
-                        path, lockspace) < 0)
-            return -1;
-        if (VIR_STRDUP(newName, name) < 0)
-            goto error;
 
-    }   break;
+            if (STRPREFIX(name, "/dev") &&
+                driver->scsiLockSpaceDir) {
+                VIR_DEBUG("Trying to find an SCSI ID for %s", name);
+                if (virStorageFileGetSCSIKey(name, &newName) < 0)
+                    goto cleanup;
+
+                if (newName) {
+                    VIR_DEBUG("Got an SCSI ID %s for %s", newName, name);
+                    if (VIR_STRDUP(newLockspace, driver->scsiLockSpaceDir) < 0)
+                        goto cleanup;
+                    newFlags |= VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_AUTOCREATE;
+                    break;
+                }
+                virResetLastError();
+                /* Fallback to generic non-block code */
+            }
+
+            if (driver->fileLockSpaceDir) {
+                if (VIR_STRDUP(newLockspace, driver->fileLockSpaceDir) < 0)
+                    goto cleanup;
+                if (virCryptoHashString(VIR_CRYPTO_HASH_SHA256, name, &newName) < 0)
+                    goto cleanup;
+                newFlags |= VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_AUTOCREATE;
+                VIR_DEBUG("Using indirect lease %s for %s", newName, name);
+            } else {
+                if (VIR_STRDUP(newLockspace, "") < 0)
+                    goto cleanup;
+                if (VIR_STRDUP(newName, name) < 0)
+                    goto cleanup;
+                VIR_DEBUG("Using direct lease for %s", name);
+            }
+
+            break;
+        case VIR_LOCK_MANAGER_RESOURCE_TYPE_LEASE: {
+            size_t i;
+            char *path = NULL;
+            char *lockspace = NULL;
+            for (i = 0; i < nparams; i++) {
+                if (STREQ(params[i].key, "offset")) {
+                    if (params[i].value.ul != 0) {
+                        virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                                       _("Offset must be zero for this lock manager"));
+                        goto cleanup;
+                    }
+                } else if (STREQ(params[i].key, "lockspace")) {
+                    lockspace = params[i].value.str;
+                } else if (STREQ(params[i].key, "path")) {
+                    path = params[i].value.str;
+                } else {
+                    virReportError(VIR_ERR_INTERNAL_ERROR,
+                                   _("Unexpected parameter %s for lease resource"),
+                                   params[i].key);
+                    goto cleanup;
+                }
+            }
+            if (!path || !lockspace) {
+                virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                               _("Missing path or lockspace for lease resource"));
+                goto cleanup;
+            }
+            if (virAsprintf(&newLockspace, "%s/%s",
+                            path, lockspace) < 0)
+                goto cleanup;
+            if (VIR_STRDUP(newName, name) < 0)
+                goto cleanup;
+
+        }   break;
+
+        case VIR_LOCK_MANAGER_RESOURCE_TYPE_METADATA:
+        default:
+            virReportError(VIR_ERR_INTERNAL_ERROR,
+                           _("Unknown lock manager object type %d for domain lock object"),
+                           type);
+            goto cleanup;
+        }
+        break;
+
+    case VIR_LOCK_MANAGER_OBJECT_TYPE_DAEMON:
+        switch ((virLockManagerResourceType) type) {
+        case VIR_LOCK_MANAGER_RESOURCE_TYPE_METADATA:
+            if (params || nparams) {
+                virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                               _("Unexpected parameters for metadata resource"));
+                goto cleanup;
+            }
+            if (VIR_STRDUP(newLockspace, "") < 0 ||
+                VIR_STRDUP(newName, name) < 0)
+                goto cleanup;
+            newFlags |= VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_METADATA;
+            break;
+
+        case VIR_LOCK_MANAGER_RESOURCE_TYPE_DISK:
+        case VIR_LOCK_MANAGER_RESOURCE_TYPE_LEASE:
+        default:
+            virReportError(VIR_ERR_INTERNAL_ERROR,
+                           _("Unknown lock manager object type %d for daemon lock object"),
+                           type);
+            goto cleanup;
+        }
+        break;
+
     default:
         virReportError(VIR_ERR_INTERNAL_ERROR,
                        _("Unknown lock manager object type %d"),
                        type);
-        return -1;
+        goto cleanup;
     }
 
-    if (VIR_EXPAND_N(priv->resources, priv->nresources, 1) < 0)
-        goto error;
-
-    priv->resources[priv->nresources-1].lockspace = newLockspace;
-    priv->resources[priv->nresources-1].name = newName;
-
     if (flags & VIR_LOCK_MANAGER_RESOURCE_SHARED)
-        priv->resources[priv->nresources-1].flags |=
-            VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_SHARED;
+        newFlags |= VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_SHARED;
 
-    if (autoCreate)
-        priv->resources[priv->nresources-1].flags |=
-            VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_AUTOCREATE;
+    if (VIR_EXPAND_N(priv->resources, priv->nresources, 1) < 0)
+        goto cleanup;
 
-    return 0;
+    VIR_STEAL_PTR(priv->resources[priv->nresources-1].lockspace, newLockspace);
+    VIR_STEAL_PTR(priv->resources[priv->nresources-1].name, newName);
+    priv->resources[priv->nresources-1].flags = newFlags;
 
- error:
+    ret = 0;
+ cleanup:
     VIR_FREE(newLockspace);
     VIR_FREE(newName);
-    return -1;
+    return ret;
+}
+
+
+static int virLockManagerLockDaemonReleaseImpl(virNetClientPtr client,
+                                               virNetClientProgramPtr program,
+                                               int counter,
+                                               virLockManagerLockDaemonResourcePtr res)
+{
+    virLockSpaceProtocolReleaseResourceArgs args;
+
+    memset(&args, 0, sizeof(args));
+
+    args.path = res->lockspace;
+    args.name = res->name;
+    args.flags = res->flags;
+
+    args.flags &=
+        ~(VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_SHARED |
+          VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_AUTOCREATE |
+          VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_METADATA);
+
+    return virNetClientProgramCall(program,
+                                   client,
+                                   counter,
+                                   VIR_LOCK_SPACE_PROTOCOL_PROC_RELEASE_RESOURCE,
+                                   0, NULL, NULL, NULL,
+                                   (xdrproc_t)xdr_virLockSpaceProtocolReleaseResourceArgs, &args,
+                                   (xdrproc_t)xdr_void, NULL);
 }
 
 
@@ -621,13 +775,17 @@ static int virLockManagerLockDaemonAcquire(virLockManagerPtr lock,
     virNetClientProgramPtr program = NULL;
     int counter = 0;
     int rv = -1;
+    ssize_t i;
+    ssize_t lastGood = -1;
     virLockManagerLockDaemonPrivatePtr priv = lock->privateData;
 
     virCheckFlags(VIR_LOCK_MANAGER_ACQUIRE_REGISTER_ONLY |
-                  VIR_LOCK_MANAGER_ACQUIRE_RESTRICT, -1);
+                  VIR_LOCK_MANAGER_ACQUIRE_RESTRICT |
+                  VIR_LOCK_MANAGER_ACQUIRE_ROLLBACK, -1);
 
-    if (priv->nresources == 0 &&
-        priv->hasRWDisks &&
+    if (priv->type == VIR_LOCK_MANAGER_OBJECT_TYPE_DOMAIN &&
+        priv->nresources == 0 &&
+        priv->t.dom.hasRWDisks &&
         driver->requireLeaseForDisks) {
         virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
                        _("Read/write, exclusive access, disks were present, but no leases specified"));
@@ -642,14 +800,12 @@ static int virLockManagerLockDaemonAcquire(virLockManagerPtr lock,
         goto cleanup;
 
     if (!(flags & VIR_LOCK_MANAGER_ACQUIRE_REGISTER_ONLY)) {
-        size_t i;
         for (i = 0; i < priv->nresources; i++) {
             virLockSpaceProtocolAcquireResourceArgs args;
 
             memset(&args, 0, sizeof(args));
 
-            if (priv->resources[i].lockspace)
-                args.path = priv->resources[i].lockspace;
+            args.path = priv->resources[i].lockspace;
             args.name = priv->resources[i].name;
             args.flags = priv->resources[i].flags;
 
@@ -661,6 +817,7 @@ static int virLockManagerLockDaemonAcquire(virLockManagerPtr lock,
                                         (xdrproc_t)xdr_virLockSpaceProtocolAcquireResourceArgs, &args,
                                         (xdrproc_t)xdr_void, NULL) < 0)
                 goto cleanup;
+            lastGood = i;
         }
     }
 
@@ -671,8 +828,28 @@ static int virLockManagerLockDaemonAcquire(virLockManagerPtr lock,
     rv = 0;
 
  cleanup:
-    if (rv != 0 && fd)
-        VIR_FORCE_CLOSE(*fd);
+    if (rv < 0) {
+        int saved_errno = errno;
+        virErrorPtr origerr;
+
+        virErrorPreserveLast(&origerr);
+        if (fd)
+            VIR_FORCE_CLOSE(*fd);
+
+        if (flags & VIR_LOCK_MANAGER_ACQUIRE_ROLLBACK) {
+            for (i = lastGood; i >= 0; i--) {
+                virLockManagerLockDaemonResourcePtr res = &priv->resources[i];
+
+                if (virLockManagerLockDaemonReleaseImpl(client, program,
+                                                        counter++, res) < 0)
+                    VIR_WARN("Unable to release resource lockspace=%s name=%s",
+                             res->lockspace, res->name);
+            }
+        }
+
+        virErrorRestore(&origerr);
+        errno = saved_errno;
+    }
     virNetClientClose(client);
     virObjectUnref(client);
     virObjectUnref(program);
@@ -700,26 +877,10 @@ static int virLockManagerLockDaemonRelease(virLockManagerPtr lock,
         goto cleanup;
 
     for (i = 0; i < priv->nresources; i++) {
-        virLockSpaceProtocolReleaseResourceArgs args;
+        virLockManagerLockDaemonResourcePtr res = &priv->resources[i];
 
-        memset(&args, 0, sizeof(args));
-
-        if (priv->resources[i].lockspace)
-            args.path = priv->resources[i].lockspace;
-        args.name = priv->resources[i].name;
-        args.flags = priv->resources[i].flags;
-
-        args.flags &=
-            ~(VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_SHARED |
-              VIR_LOCK_SPACE_PROTOCOL_ACQUIRE_RESOURCE_AUTOCREATE);
-
-        if (virNetClientProgramCall(program,
-                                    client,
-                                    counter++,
-                                    VIR_LOCK_SPACE_PROTOCOL_PROC_RELEASE_RESOURCE,
-                                    0, NULL, NULL, NULL,
-                                    (xdrproc_t)xdr_virLockSpaceProtocolReleaseResourceArgs, &args,
-                                    (xdrproc_t)xdr_void, NULL) < 0)
+        if (virLockManagerLockDaemonReleaseImpl(client, program,
+                                                counter++, res) < 0)
             goto cleanup;
     }
 
