@@ -25,8 +25,6 @@
 
 #include <config.h>
 
-#include <stdio.h>
-#include <stdlib.h>
 
 #include "virthread.h"
 #include "virfile.h"
