@@ -26,8 +26,6 @@
 #include <config.h>
 
 #include <time.h>
-#include <stdio.h>
-#include <stdlib.h>
 
 #if WITH_AVAHI
 # include <avahi-client/client.h>
