@@ -25,12 +25,8 @@
 
 #include <config.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <stdarg.h>
 #include <unistd.h>
-#include <errno.h>
 #include <sys/types.h>
 #include <fcntl.h>
 #include <getopt.h>
