@@ -25,9 +25,7 @@
 
 #include <sys/wait.h>
 #include <sys/stat.h>
-#include <stdio.h>
 #include <regex.h>
-#include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
 
