@@ -29,8 +29,6 @@
 
 #include <unistd.h>
 #include <fcntl.h>
-#include <stdio.h>
-#include <stdlib.h>
 
 #include "virutil.h"
 #include "virthread.h"
