@@ -23,8 +23,6 @@
 
 #include <config.h>
 
-#include <stdlib.h>
-#include <stdio.h>
 #include <arpa/inet.h>
 #ifdef __linux__
 # include <linux/rtnetlink.h>
@@ -152,6 +150,8 @@ virArpTableGet(void)
         }
     }
 
+    return table;
+
  cleanup:
     virArpTableFree(table);
     return NULL;
