@@ -22,8 +22,6 @@
 
 #include <config.h>
 
-#include <string.h>
-#include <stdio.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -296,7 +294,9 @@ virConfSaveValue(virBufferPtr buf, virConfValuePtr val)
             virBufferAddLit(buf, " ]");
             break;
         }
+        case VIR_CONF_LAST:
         default:
+            virReportEnumRangeError(virConfType, val->type);
             return -1;
     }
     return 0;
@@ -461,7 +461,7 @@ virConfParseValue(virConfParserCtxtPtr ctxt)
         return NULL;
     }
     if ((CUR == '"') || (CUR == '\'') ||
-         (ctxt->conf->flags & VIR_CONF_FLAG_LXC_FORMAT)) {
+        (ctxt->conf->flags & VIR_CONF_FLAG_LXC_FORMAT)) {
         type = VIR_CONF_STRING;
         str = virConfParseString(ctxt);
         if (str == NULL)
@@ -986,13 +986,20 @@ int virConfGetValueStringList(virConfPtr conf,
         }
         ATTRIBUTE_FALLTHROUGH;
 
-    default:
+    case VIR_CONF_LLONG:
+    case VIR_CONF_ULLONG:
+    case VIR_CONF_NONE:
         virReportError(VIR_ERR_INTERNAL_ERROR,
                        compatString ?
                        _("%s: expected a string or string list for '%s' parameter") :
                        _("%s: expected a string list for '%s' parameter"),
                        conf->filename, setting);
         return -1;
+
+    case VIR_CONF_LAST:
+    default:
+        virReportEnumRangeError(virConfType, cval->type);
+        return -1;
     }
 
     return 1;
@@ -1419,7 +1426,7 @@ int virConfWalk(virConfPtr conf,
     cur = conf->entries;
     while (cur != NULL) {
         if (cur->name && cur->value &&
-                callback(cur->name, cur->value, opaque) < 0)
+            callback(cur->name, cur->value, opaque) < 0)
             return -1;
         cur = cur->next;
     }
