@@ -35,9 +35,6 @@
 #include "virstring.h"
 #include "datatypes.h"
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
 #include <unistd.h>
 #include <regex.h>
 #include <sys/types.h>
