@@ -46,12 +46,12 @@ virMutex virNetDevVethCreateMutex = VIR_MUTEX_INITIALIZER;
 static int virNetDevVethExists(int devNum)
 {
     int ret;
-    char *path = NULL;
+    VIR_AUTOFREE(char *) path = NULL;
+
     if (virAsprintf(&path, SYSFS_NET_DIR "vnet%d/", devNum) < 0)
         return -1;
     ret = virFileExists(path) ? 1 : 0;
     VIR_DEBUG("Checked dev vnet%d usage: %d", devNum, ret);
-    VIR_FREE(path);
     return ret;
 }
 
@@ -111,10 +111,7 @@ static int virNetDevVethGetFreeNum(int startDev)
 int virNetDevVethCreate(char** veth1, char** veth2)
 {
     int ret = -1;
-    char *veth1auto = NULL;
-    char *veth2auto = NULL;
     int vethNum = 0;
-    virCommandPtr cmd = NULL;
     size_t i;
 
     /*
@@ -125,6 +122,10 @@ int virNetDevVethCreate(char** veth1, char** veth2)
 #define MAX_VETH_RETRIES 10
 
     for (i = 0; i < MAX_VETH_RETRIES; i++) {
+        VIR_AUTOFREE(char *) veth1auto = NULL;
+        VIR_AUTOFREE(char *) veth2auto = NULL;
+        VIR_AUTOPTR(virCommand) cmd = NULL;
+
         int status;
         if (!*veth1) {
             int veth1num;
@@ -173,10 +174,6 @@ int virNetDevVethCreate(char** veth1, char** veth2)
                   *veth1 ? *veth1 : veth1auto,
                   *veth2 ? *veth2 : veth2auto,
                   status);
-        VIR_FREE(veth1auto);
-        VIR_FREE(veth2auto);
-        virCommandFree(cmd);
-        cmd = NULL;
     }
 
     virReportError(VIR_ERR_INTERNAL_ERROR,
@@ -185,9 +182,6 @@ int virNetDevVethCreate(char** veth1, char** veth2)
 
  cleanup:
     virMutexUnlock(&virNetDevVethCreateMutex);
-    virCommandFree(cmd);
-    VIR_FREE(veth1auto);
-    VIR_FREE(veth2auto);
     return ret;
 }
 
@@ -204,26 +198,22 @@ int virNetDevVethCreate(char** veth1, char** veth2)
  */
 int virNetDevVethDelete(const char *veth)
 {
-    virCommandPtr cmd = virCommandNewArgList("ip", "link", "del", veth, NULL);
     int status;
-    int ret = -1;
+    VIR_AUTOPTR(virCommand) cmd = virCommandNewArgList("ip", "link",
+                                                       "del", veth, NULL);
 
     if (virCommandRun(cmd, &status) < 0)
-        goto cleanup;
+        return -1;
 
     if (status != 0) {
         if (!virNetDevExists(veth)) {
             VIR_DEBUG("Device %s already deleted (by kernel namespace cleanup)", veth);
-            ret = 0;
-            goto cleanup;
+            return 0;
         }
         virReportError(VIR_ERR_INTERNAL_ERROR,
                        _("Failed to delete veth device %s"), veth);
-        goto cleanup;
+        return -1;
     }
 
-    ret = 0;
- cleanup:
-    virCommandFree(cmd);
-    return ret;
+    return 0;
 }
