@@ -30,7 +30,6 @@
 
 #include <config.h>
 
-#include <errno.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -488,6 +487,117 @@ virNetlinkDumpLink(const char *ifname, int ifindex,
 }
 
 
+/**
+ * virNetlinkNewLink:
+ *
+ * @ifname: name of the link
+ * @type: the type of the device, i.e. "bridge", "macvtap", "macvlan"
+ * @extra_args: the extra args for creating the netlink interface
+ * @error: netlink error code
+ *
+ * A generic wrapper to create a network link.
+ *
+ * Returns 0 on success, -1 on error. Additionally, if the @error is
+ * non-zero, then a failure occurred during virNetlinkCommand, but
+ * no error message is generated leaving it up to the caller to handle
+ * the condition.
+ */
+int
+virNetlinkNewLink(const char *ifname,
+                  const char *type,
+                  virNetlinkNewLinkDataPtr extra_args,
+                  int *error)
+{
+    struct nlmsgerr *err;
+    struct nlattr *linkinfo = NULL;
+    struct nlattr *infodata = NULL;
+    unsigned int buflen;
+    struct ifinfomsg ifinfo = { .ifi_family = AF_UNSPEC };
+    VIR_AUTOPTR(virNetlinkMsg) nl_msg = NULL;
+    VIR_AUTOFREE(struct nlmsghdr *) resp = NULL;
+
+    *error = 0;
+
+    VIR_DEBUG("Creating %s interface '%s'", type, ifname);
+
+    if (!ifname || !type) {
+        virReportError(VIR_ERR_INVALID_ARG, "%s",
+                       _("both interface name and type must not be NULL"));
+        return -1;
+    }
+
+    nl_msg = nlmsg_alloc_simple(RTM_NEWLINK,
+                                NLM_F_REQUEST | NLM_F_CREATE | NLM_F_EXCL);
+    if (!nl_msg) {
+        virReportOOMError();
+        return -1;
+    }
+
+    if (nlmsg_append(nl_msg,  &ifinfo, sizeof(ifinfo), NLMSG_ALIGNTO) < 0)
+        goto buffer_too_small;
+
+    NETLINK_MSG_PUT(nl_msg, IFLA_IFNAME, (strlen(ifname) + 1), ifname);
+
+    NETLINK_MSG_NEST_START(nl_msg, linkinfo, IFLA_LINKINFO);
+    NETLINK_MSG_PUT(nl_msg, IFLA_INFO_KIND, (strlen(type) + 1), type);
+
+    if ((STREQ(type, "macvtap") || STREQ(type, "macvlan")) &&
+        extra_args &&
+        extra_args->macvlan_mode &&
+        *extra_args->macvlan_mode > 0) {
+        NETLINK_MSG_NEST_START(nl_msg, infodata, IFLA_INFO_DATA);
+        NETLINK_MSG_PUT(nl_msg, IFLA_MACVLAN_MODE,
+                        sizeof(uint32_t), extra_args->macvlan_mode);
+        NETLINK_MSG_NEST_END(nl_msg, infodata);
+    }
+
+    NETLINK_MSG_NEST_END(nl_msg, linkinfo);
+
+    if (extra_args) {
+        NETLINK_MSG_PUT(nl_msg, IFLA_LINK,
+                        sizeof(uint32_t), extra_args->ifindex);
+        NETLINK_MSG_PUT(nl_msg, IFLA_ADDRESS, VIR_MAC_BUFLEN, extra_args->mac);
+    }
+
+    if (virNetlinkCommand(nl_msg, &resp, &buflen, 0, 0, NETLINK_ROUTE, 0) < 0)
+        return -1;
+
+    if (buflen < NLMSG_LENGTH(0) || resp == NULL)
+        goto malformed_resp;
+
+    switch (resp->nlmsg_type) {
+    case NLMSG_ERROR:
+        err = (struct nlmsgerr *)NLMSG_DATA(resp);
+        if (resp->nlmsg_len < NLMSG_LENGTH(sizeof(*err)))
+            goto malformed_resp;
+
+        if (err->error < 0) {
+            *error = err->error;
+            return -1;
+        }
+        break;
+
+    case NLMSG_DONE:
+        break;
+
+    default:
+        goto malformed_resp;
+    }
+
+    return 0;
+
+ malformed_resp:
+    virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                   _("malformed netlink response message"));
+    return -1;
+
+ buffer_too_small:
+    virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                   _("allocated netlink buffer is too small"));
+    return -1;
+}
+
+
 /**
  * virNetlinkDelLink:
  *
@@ -760,7 +870,7 @@ virNetlinkEventCallback(int watch,
     virNetlinkEventServerLock(srv);
 
     VIR_DEBUG("dispatching to max %d clients, called from event watch %d",
-            (int)srv->handlesCount, watch);
+              (int)srv->handlesCount, watch);
 
     for (i = 0; i < srv->handlesCount; i++) {
         if (srv->handles[i].deleted != VIR_NETLINK_HANDLE_VALID)
@@ -1032,7 +1142,7 @@ virNetlinkEventAddClient(virNetlinkEventHandleCallback handleCB,
         VIR_DEBUG("Used %zu handle slots, adding at least %d more",
                   srv->handlesAlloc, NETLINK_EVENT_ALLOC_EXTENT);
         if (VIR_RESIZE_N(srv->handles, srv->handlesAlloc,
-                        srv->handlesCount, NETLINK_EVENT_ALLOC_EXTENT) < 0)
+                         srv->handlesCount, NETLINK_EVENT_ALLOC_EXTENT) < 0)
             goto error;
     }
     r = srv->handlesCount++;
@@ -1182,6 +1292,17 @@ virNetlinkDelLink(const char *ifname ATTRIBUTE_UNUSED,
 }
 
 
+int
+virNetlinkNewLink(const char *ifname ATTRIBUTE_UNUSED,
+                  const char *type ATTRIBUTE_UNUSED,
+                  virNetlinkNewLinkDataPtr extra_args ATTRIBUTE_UNUSED,
+                  int *error ATTRIBUTE_UNUSED)
+{
+    virReportError(VIR_ERR_INTERNAL_ERROR, "%s", _(unsupported));
+    return -1;
+}
+
+
 int
 virNetlinkGetNeighbor(void **nlData ATTRIBUTE_UNUSED,
                       uint32_t src_pid ATTRIBUTE_UNUSED,
