@@ -24,8 +24,6 @@
 
 #include <fcntl.h>
 #include <signal.h>
-#include <errno.h>
-#include <stdlib.h>
 #include <sys/wait.h>
 #include <unistd.h>
 #if HAVE_SYS_MOUNT_H
@@ -702,7 +700,7 @@ int virProcessSetNamespaces(size_t nfdlist,
 
     if (nfdlist == 0) {
         virReportInvalidArg(nfdlist, "%s",
-                             _("Expected at least one file descriptor"));
+                            _("Expected at least one file descriptor"));
         return -1;
     }
     for (i = 0; i < nfdlist; i++) {
