@@ -43,9 +43,8 @@ int virQEMUBuildCommandLineJSON(virJSONValuePtr value,
                                 virBufferPtr buf,
                                 virQEMUBuildCommandLineJSONArrayFormatFunc array);
 
-char *virQEMUBuildObjectCommandlineFromJSON(const char *type,
-                                            const char *alias,
-                                            virJSONValuePtr props);
+int virQEMUBuildObjectCommandlineFromJSON(virBufferPtr buf,
+                                          virJSONValuePtr objprops);
 
 char *virQEMUBuildDriveCommandlineFromJSON(virJSONValuePtr src);
 
