@@ -70,6 +70,12 @@ VIR_ENUM_IMPL(virResctrl, VIR_CACHE_TYPE_LAST,
               "CODE",
               "DATA")
 
+/* Monitor feature name prefix mapping for monitor naming */
+VIR_ENUM_IMPL(virResctrlMonitorPrefix, VIR_RESCTRL_MONITOR_TYPE_LAST,
+              "__unsupported__",
+              "llc_",
+              "mbm_")
+
 
 /* All private typedefs so that they exist for all later definitions.  This way
  * structs can be included in one or another without reorganizing the code every
@@ -83,6 +89,9 @@ typedef virResctrlInfoPerLevel *virResctrlInfoPerLevelPtr;
 typedef struct _virResctrlInfoMemBW virResctrlInfoMemBW;
 typedef virResctrlInfoMemBW *virResctrlInfoMemBWPtr;
 
+typedef struct _virResctrlInfoMongrp virResctrlInfoMongrp;
+typedef virResctrlInfoMongrp *virResctrlInfoMongrpPtr;
+
 typedef struct _virResctrlAllocPerType virResctrlAllocPerType;
 typedef virResctrlAllocPerType *virResctrlAllocPerTypePtr;
 
@@ -139,6 +148,28 @@ struct _virResctrlInfoMemBW {
     unsigned int max_id;
 };
 
+struct _virResctrlInfoMongrp {
+    /* Maximum number of simultaneous monitors */
+    unsigned int max_monitor;
+    /* null-terminal string list for monitor features */
+    char **features;
+    /* Number of monitor features */
+    size_t nfeatures;
+
+    /* Last level cache related information */
+
+    /* This adjustable value affects the final reuse of resources used by
+     * monitor. After the action of removing a monitor, the kernel may not
+     * release all hardware resources that monitor used immediately if the
+     * cache occupancy value associated with 'removed' monitor is above this
+     * threshold. Once the cache occupancy is below this threshold, the
+     * underlying hardware resource will be reclaimed and be put into the
+     * resource pool for next reusing.*/
+    unsigned int cache_reuse_threshold;
+    /* The cache 'level' that has the monitor capability */
+    unsigned int cache_level;
+};
+
 struct _virResctrlInfo {
     virObject parent;
 
@@ -146,6 +177,8 @@ struct _virResctrlInfo {
     size_t nlevels;
 
     virResctrlInfoMemBWPtr membw_info;
+
+    virResctrlInfoMongrpPtr monitor_info;
 };
 
 
@@ -171,8 +204,23 @@ virResctrlInfoDispose(void *obj)
         VIR_FREE(level);
     }
 
+    if (resctrl->monitor_info)
+        virStringListFree(resctrl->monitor_info->features);
+
     VIR_FREE(resctrl->membw_info);
     VIR_FREE(resctrl->levels);
+    VIR_FREE(resctrl->monitor_info);
+}
+
+
+void
+virResctrlInfoMonFree(virResctrlInfoMonPtr mon)
+{
+    if (!mon)
+        return;
+
+    virStringListFree(mon->features);
+    VIR_FREE(mon);
 }
 
 
@@ -555,6 +603,94 @@ virResctrlGetMemoryBandwidthInfo(virResctrlInfoPtr resctrl)
 }
 
 
+/*
+ * Retrieve monitor capability from the resource control file system.
+ *
+ * The monitor capability is exposed through "SYSFS_RESCTRL_PATH/info/L3_MON"
+ * directory under the resource control file system. The monitor capability is
+ * parsed by reading the interface files and stored in the structure
+ * 'virResctrlInfoMongrp'.
+ *
+ * Not all host supports the resource monitor, leave the pointer
+ * @resctrl->monitor_info empty if not supported.
+ */
+static int
+virResctrlGetMonitorInfo(virResctrlInfoPtr resctrl)
+{
+    int ret = -1;
+    int rv = -1;
+    char *featurestr = NULL;
+    char **features = NULL;
+    size_t nfeatures = 0;
+    virResctrlInfoMongrpPtr info_monitor = NULL;
+
+    if (VIR_ALLOC(info_monitor) < 0)
+        return -1;
+
+    /* For now, monitor only exists in level 3 cache */
+    info_monitor->cache_level = 3;
+
+    rv = virFileReadValueUint(&info_monitor->max_monitor,
+                              SYSFS_RESCTRL_PATH "/info/L3_MON/num_rmids");
+    if (rv == -2) {
+        /* The file doesn't exist, so it's unusable for us, probably resource
+         * monitor unsupported */
+        VIR_INFO("The file '" SYSFS_RESCTRL_PATH "/info/L3_MON/num_rmids' "
+                 "does not exist");
+        ret = 0;
+        virResetLastError();
+        goto cleanup;
+    } else if (rv < 0) {
+        /* Other failures are fatal, so just quit */
+        goto cleanup;
+    }
+
+    rv = virFileReadValueUint(&info_monitor->cache_reuse_threshold,
+                              SYSFS_RESCTRL_PATH
+                              "/info/L3_MON/max_threshold_occupancy");
+    if (rv == -2) {
+        /* If CMT is not supported, then 'max_threshold_occupancy' file
+         * will not exist. */
+        VIR_DEBUG("File '" SYSFS_RESCTRL_PATH
+                  "/info/L3_MON/max_threshold_occupancy' does not exist");
+        virResetLastError();
+    } else if (rv < 0) {
+        goto cleanup;
+    }
+
+    rv = virFileReadValueString(&featurestr,
+                                SYSFS_RESCTRL_PATH
+                                "/info/L3_MON/mon_features");
+    if (rv == -2)
+        virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                       _("Cannot get mon_features from resctrl"));
+    if (rv < 0)
+        goto cleanup;
+
+    if (!*featurestr) {
+        /* If no feature found in "/info/L3_MON/mon_features",
+         * some error happens */
+        virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                       _("Get empty feature list from resctrl"));
+        goto cleanup;
+    }
+
+    features = virStringSplitCount(featurestr, "\n", 0, &nfeatures);
+    VIR_DEBUG("Resctrl supported %zd monitoring features", nfeatures);
+
+    info_monitor->nfeatures = nfeatures;
+    VIR_STEAL_PTR(info_monitor->features, features);
+    VIR_STEAL_PTR(resctrl->monitor_info, info_monitor);
+
+    ret = 0;
+ cleanup:
+    VIR_FREE(featurestr);
+    virStringListFree(features);
+    VIR_FREE(info_monitor);
+    return ret;
+}
+
+
 static int
 virResctrlGetInfo(virResctrlInfoPtr resctrl)
 {
@@ -573,6 +709,10 @@ virResctrlGetInfo(virResctrlInfoPtr resctrl)
     if (ret < 0)
         goto cleanup;
 
+    ret = virResctrlGetMonitorInfo(resctrl);
+    if (ret < 0)
+        goto cleanup;
+
     ret = 0;
  cleanup:
     VIR_DIR_CLOSE(dirp);
@@ -613,6 +753,9 @@ virResctrlInfoIsEmpty(virResctrlInfoPtr resctrl)
     if (resctrl->membw_info)
         return false;
 
+    if (resctrl->monitor_info)
+        return false;
+
     for (i = 0; i < resctrl->nlevels; i++) {
         virResctrlInfoPerLevelPtr i_level = resctrl->levels[i];
 
@@ -702,7 +845,7 @@ virResctrlInfoGetCache(virResctrlInfoPtr resctrl,
                 virReportError(VIR_ERR_INTERNAL_ERROR,
                                _("level %u cache size %llu does not match "
                                  "expected size %llu"),
-                                 level, i_type->size, size);
+                               level, i_type->size, size);
                 goto error;
             }
             i_type->max_cache_id++;
@@ -727,6 +870,99 @@ virResctrlInfoGetCache(virResctrlInfoPtr resctrl,
 }
 
 
+/* virResctrlInfoGetMonitorPrefix
+ *
+ * @resctrl: Pointer to virResctrlInfo
+ * @prefix: Monitor prefix name for monitor looking for.
+ * @monitor: Returns the capability information for target monitor if the
+ * monitor with @prefex is supported by host.
+ *
+ * Return monitor capability information for @prefix through @monitor.
+ * If monitor with @prefix is not supported in system, @monitor will be
+ * cleared to NULL.
+ *
+ * Returns 0 if @monitor is created or monitor type with @prefix is not
+ * supported by host, -1 on failure with error message set.
+ */
+int
+virResctrlInfoGetMonitorPrefix(virResctrlInfoPtr resctrl,
+                               const char *prefix,
+                               virResctrlInfoMonPtr *monitor)
+{
+    size_t i = 0;
+    virResctrlInfoMongrpPtr mongrp_info = NULL;
+    virResctrlInfoMonPtr mon = NULL;
+    int ret = -1;
+
+    if (!prefix) {
+        virReportError(VIR_ERR_INTERNAL_ERROR, "%s",
+                       _("Empty prefix name for resctrl monitor"));
+        return -1;
+    }
+
+    if (virResctrlInfoIsEmpty(resctrl))
+        return 0;
+
+    mongrp_info = resctrl->monitor_info;
+
+    if (!mongrp_info) {
+        VIR_INFO("Monitor is not supported in host");
+        return 0;
+    }
+
+    for (i = 0; i < VIR_RESCTRL_MONITOR_TYPE_LAST; i++) {
+        if (STREQ(prefix, virResctrlMonitorPrefixTypeToString(i))) {
+            if (VIR_ALLOC(mon) < 0)
+                goto cleanup;
+            mon->type = i;
+            break;
+        }
+    }
+
+    if (!mon) {
+        virReportError(VIR_ERR_INTERNAL_ERROR,
+                       _("Bad prefix name '%s' for resctrl monitor"),
+                       prefix);
+        return -1;
+    }
+
+    mon->max_monitor = mongrp_info->max_monitor;
+
+    if (mon->type == VIR_RESCTRL_MONITOR_TYPE_CACHE) {
+        mon->cache_reuse_threshold =  mongrp_info->cache_reuse_threshold;
+        mon->cache_level = mongrp_info->cache_level;
+    }
+
+    for (i = 0; i < mongrp_info->nfeatures; i++) {
+        if (STRPREFIX(mongrp_info->features[i], prefix)) {
+            if (virStringListAdd(&mon->features,
+                                 mongrp_info->features[i]) < 0)
+                goto cleanup;
+            mon->nfeatures++;
+        }
+    }
+
+    ret = 0;
+
+    /* In case *monitor is pointed to some monitor, clean it. */
+    virResctrlInfoMonFree(*monitor);
+
+    if (mon->nfeatures == 0) {
+        /* No feature found for current monitor, means host does not support
+         * monitor type with @prefix name.
+         * Telling caller this monitor is supported by hardware specification,
+         * but not supported by this host. */
+        VIR_INFO("No resctrl monitor features using prefix '%s' found", prefix);
+        goto cleanup;
+    }
+
+    VIR_STEAL_PTR(*monitor, mon);
+ cleanup:
+    virResctrlInfoMonFree(mon);
+    return ret;
+}
+
+
 /* virResctrlAlloc-related definitions */
 virResctrlAllocPtr
 virResctrlAllocNew(void)
@@ -1430,9 +1666,8 @@ virResctrlAllocGetGroup(virResctrlInfoPtr resctrl,
 {
     char *schemata = NULL;
     int rv = virFileReadValueString(&schemata,
-                                     SYSFS_RESCTRL_PATH
-                                     "/%s/schemata",
-                                     groupname);
+                                    SYSFS_RESCTRL_PATH "/%s/schemata",
+                                    groupname);
 
     *alloc = NULL;
 
