@@ -25,11 +25,7 @@
 
 #include "viruuid.h"
 
-#include <errno.h>
 #include <fcntl.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <time.h>
