@@ -22,8 +22,6 @@
 
 #include <config.h>
 
-#include <limits.h>
-#include <string.h>
 #include <curl/curl.h>
 #include <xen/api/xen_all.h>
 #include "internal.h"
@@ -447,7 +445,6 @@ xenapiNodeGetInfo(virConnectPtr conn, virNodeInfoPtr info)
 static char *
 xenapiConnectGetCapabilities(virConnectPtr conn)
 {
-
     virCapsPtr caps = ((struct _xenapiPrivate *)(conn->privateData))->caps;
     if (caps)
         return virCapabilitiesFormatXML(caps);
@@ -1241,7 +1238,6 @@ xenapiDomainGetVcpus(virDomainPtr dom,
                      virVcpuInfoPtr info, int maxinfo,
                      unsigned char *cpumaps, int maplen)
 {
-
     xen_vm_set *vms = NULL;
     xen_vm vm = NULL;
     xen_string_string_map *vcpu_params = NULL;
