@@ -44,7 +44,7 @@ xenParseXMOS(virConfPtr conf, virDomainDefPtr def)
     size_t i;
 
     if (def->os.type == VIR_DOMAIN_OSTYPE_HVM) {
-        const char *boot;
+        VIR_AUTOFREE(char *) boot = NULL;
 
         if (VIR_ALLOC(def->os.loader) < 0 ||
             xenConfigCopyString(conf, "kernel", &def->os.loader->path) < 0)
@@ -72,7 +72,8 @@ xenParseXMOS(virConfPtr conf, virDomainDefPtr def)
             def->os.nBootDevs++;
         }
     } else {
-        const char *extra, *root;
+        VIR_AUTOFREE(char *) extra = NULL;
+        VIR_AUTOFREE(char *) root = NULL;
 
         if (xenConfigCopyStringOpt(conf, "bootloader", &def->os.bootloader) < 0)
             return -1;
@@ -417,7 +418,7 @@ xenFormatXMDisks(virConfPtr conf, virDomainDefPtr def)
 static int
 xenParseXMInputDevs(virConfPtr conf, virDomainDefPtr def)
 {
-    const char *str;
+    VIR_AUTOFREE(char *) str = NULL;
 
     if (def->os.type == VIR_DOMAIN_OSTYPE_HVM) {
         if (xenConfigGetString(conf, "usbdevice", &str, NULL) < 0)
