@@ -20,9 +20,6 @@
 
 #include <config.h>
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <time.h>
 
 #include <selinux/selinux.h>
@@ -275,7 +272,7 @@ mymain(void)
     int ret = 0;
     virSecurityManagerPtr mgr;
 
-    if (!(mgr = virSecurityManagerNew("selinux", "QEMU",
+    if (!(mgr = virSecurityManagerNew("selinux", "QEMU", NULL,
                                       VIR_SECURITY_MANAGER_DEFAULT_CONFINED |
                                       VIR_SECURITY_MANAGER_PRIVILEGED))) {
         fprintf(stderr, "Unable to initialize security driver: %s\n",
