@@ -85,7 +85,7 @@ testCompareXMLToArgvFiles(bool shouldFail,
      * convert the inputvol afterwards. Since we only care about the
      * command line we have to copy code from storageBackendCreateQemuImg
      * and adjust it for the test needs. */
-    if (inputvol && vol->target.encryption)
+    if (inputvol && (vol->target.encryption || inputvol->target.encryption))
         convertStep = VIR_STORAGE_VOL_ENCRYPT_CREATE;
 
     do {
@@ -93,6 +93,7 @@ testCompareXMLToArgvFiles(bool shouldFail,
                                                        inputvol, flags,
                                                        create_tool,
                                                        "/path/to/secretFile",
+                                                       "/path/to/inputSecretFile",
                                                        convertStep);
         if (!cmd) {
             if (shouldFail) {
@@ -284,6 +285,22 @@ mymain(void)
             "pool-dir", "vol-file",
             "luks-convert", 0);
 
+    DO_TEST("pool-dir", "vol-luks-convert",
+            "pool-dir", "vol-file-qcow2",
+            "luks-convert-qcow2", 0);
+
+    DO_TEST("pool-dir", "vol-encrypt2",
+            "pool-dir", "vol-encrypt1",
+            "luks-convert-encrypt", 0);
+
+    DO_TEST("pool-dir", "vol-file",
+            "pool-dir", "vol-encrypt2",
+            "luks-convert-encrypt2fileraw", 0);
+
+    DO_TEST("pool-dir", "vol-file-qcow2",
+            "pool-dir", "vol-encrypt2",
+            "luks-convert-encrypt2fileqcow2", 0);
+
     return ret == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
