@@ -24,7 +24,6 @@
 
 #ifdef __linux__
 
-# include <stdlib.h>
 
 # define __VIR_CGROUP_ALLOW_INCLUDE_PRIV_H__
 # include "vircgrouppriv.h"
@@ -56,25 +55,25 @@ static int validateCgroup(virCgroupPtr cgroup,
 
     for (i = 0; i < VIR_CGROUP_CONTROLLER_LAST; i++) {
         if (STRNEQ_NULLABLE(expectMountPoint[i],
-                            cgroup->controllers[i].mountPoint)) {
+                            cgroup->legacy[i].mountPoint)) {
             fprintf(stderr, "Wrong mount '%s', expected '%s' for '%s'\n",
-                    cgroup->controllers[i].mountPoint,
+                    cgroup->legacy[i].mountPoint,
                     expectMountPoint[i],
                     virCgroupControllerTypeToString(i));
             return -1;
         }
         if (STRNEQ_NULLABLE(expectLinkPoint[i],
-                            cgroup->controllers[i].linkPoint)) {
+                            cgroup->legacy[i].linkPoint)) {
             fprintf(stderr, "Wrong link '%s', expected '%s' for '%s'\n",
-                    cgroup->controllers[i].linkPoint,
+                    cgroup->legacy[i].linkPoint,
                     expectLinkPoint[i],
                     virCgroupControllerTypeToString(i));
             return -1;
         }
         if (STRNEQ_NULLABLE(expectPlacement[i],
-                            cgroup->controllers[i].placement)) {
+                            cgroup->legacy[i].placement)) {
             fprintf(stderr, "Wrong placement '%s', expected '%s' for '%s'\n",
-                    cgroup->controllers[i].placement,
+                    cgroup->legacy[i].placement,
                     expectPlacement[i],
                     virCgroupControllerTypeToString(i));
             return -1;
@@ -114,16 +113,6 @@ const char *mountsAllInOne[VIR_CGROUP_CONTROLLER_LAST] = {
     [VIR_CGROUP_CONTROLLER_BLKIO] = "/not/really/sys/fs/cgroup",
     [VIR_CGROUP_CONTROLLER_SYSTEMD] = NULL,
 };
-const char *mountsLogind[VIR_CGROUP_CONTROLLER_LAST] = {
-    [VIR_CGROUP_CONTROLLER_CPU] = NULL,
-    [VIR_CGROUP_CONTROLLER_CPUACCT] = NULL,
-    [VIR_CGROUP_CONTROLLER_CPUSET] = NULL,
-    [VIR_CGROUP_CONTROLLER_MEMORY] = NULL,
-    [VIR_CGROUP_CONTROLLER_DEVICES] = NULL,
-    [VIR_CGROUP_CONTROLLER_FREEZER] = NULL,
-    [VIR_CGROUP_CONTROLLER_BLKIO] = NULL,
-    [VIR_CGROUP_CONTROLLER_SYSTEMD] = "/not/really/sys/fs/cgroup/systemd",
-};
 
 const char *links[VIR_CGROUP_CONTROLLER_LAST] = {
     [VIR_CGROUP_CONTROLLER_CPU] = "/not/really/sys/fs/cgroup/cpu",
@@ -147,15 +136,10 @@ const char *linksAllInOne[VIR_CGROUP_CONTROLLER_LAST] = {
     [VIR_CGROUP_CONTROLLER_SYSTEMD] = NULL,
 };
 
-const char *linksLogind[VIR_CGROUP_CONTROLLER_LAST] = {
-    [VIR_CGROUP_CONTROLLER_CPU] = NULL,
-    [VIR_CGROUP_CONTROLLER_CPUACCT] = NULL,
-    [VIR_CGROUP_CONTROLLER_CPUSET] = NULL,
-    [VIR_CGROUP_CONTROLLER_MEMORY] = NULL,
-    [VIR_CGROUP_CONTROLLER_DEVICES] = NULL,
-    [VIR_CGROUP_CONTROLLER_FREEZER] = NULL,
-    [VIR_CGROUP_CONTROLLER_BLKIO] = NULL,
-    [VIR_CGROUP_CONTROLLER_SYSTEMD] = NULL,
+
+struct _detectMountsData {
+    const char *file;
+    bool fail;
 };
 
 
@@ -163,28 +147,33 @@ static int
 testCgroupDetectMounts(const void *args)
 {
     int result = -1;
-    const char *file = args;
-    char *mounts = NULL;
+    const struct _detectMountsData *data = args;
     char *parsed = NULL;
     const char *actual;
     virCgroupPtr group = NULL;
     virBuffer buf = VIR_BUFFER_INITIALIZER;
     size_t i;
 
-    if (virAsprintf(&mounts, "%s/vircgroupdata/%s.mounts",
-                    abs_srcdir, file) < 0 ||
-        virAsprintf(&parsed, "%s/vircgroupdata/%s.parsed",
-                    abs_srcdir, file) < 0 ||
-        VIR_ALLOC(group) < 0)
-        goto cleanup;
+    setenv("VIR_CGROUP_MOCK_FILENAME", data->file, 1);
 
-    if (virCgroupDetectMountsFromFile(group, mounts, false) < 0)
+    if (virAsprintf(&parsed, "%s/vircgroupdata/%s.parsed",
+                    abs_srcdir, data->file) < 0) {
+        goto cleanup;
+    }
+
+    if (virCgroupNewSelf(&group) < 0) {
+        if (data->fail)
+            result = 0;
+        goto cleanup;
+    }
+
+    if (data->fail)
         goto cleanup;
 
     for (i = 0; i < VIR_CGROUP_CONTROLLER_LAST; i++) {
         virBufferAsprintf(&buf, "%-12s %s\n",
                           virCgroupControllerTypeToString(i),
-                          NULLSTR(group->controllers[i].mountPoint));
+                          NULLSTR(group->legacy[i].mountPoint));
     }
     if (virBufferCheckError(&buf) < 0)
         goto cleanup;
@@ -196,7 +185,7 @@ testCgroupDetectMounts(const void *args)
     result = 0;
 
  cleanup:
-    VIR_FREE(mounts);
+    unsetenv("VIR_CGROUP_MOCK_FILENAME");
     VIR_FREE(parsed);
     virCgroupFree(&group);
     virBufferFreeAndReset(&buf);
@@ -544,24 +533,13 @@ static int testCgroupNewForSelfLogind(const void *args ATTRIBUTE_UNUSED)
 {
     virCgroupPtr cgroup = NULL;
     int ret = -1;
-    const char *placement[VIR_CGROUP_CONTROLLER_LAST] = {
-        [VIR_CGROUP_CONTROLLER_CPU] = NULL,
-        [VIR_CGROUP_CONTROLLER_CPUACCT] = NULL,
-        [VIR_CGROUP_CONTROLLER_CPUSET] = NULL,
-        [VIR_CGROUP_CONTROLLER_MEMORY] = NULL,
-        [VIR_CGROUP_CONTROLLER_DEVICES] = NULL,
-        [VIR_CGROUP_CONTROLLER_FREEZER] = NULL,
-        [VIR_CGROUP_CONTROLLER_BLKIO] = NULL,
-        [VIR_CGROUP_CONTROLLER_SYSTEMD] = "/",
-    };
 
-    if (virCgroupNewSelf(&cgroup) < 0) {
-        fprintf(stderr, "Cannot create cgroup for self\n");
+    if (virCgroupNewSelf(&cgroup) == 0) {
+        fprintf(stderr, "Expected cgroup creation to fail.\n");
         goto cleanup;
     }
 
-    ret = validateCgroup(cgroup, "", mountsLogind, linksLogind, placement);
-
+    ret = 0;
  cleanup:
     virCgroupFree(&cgroup);
     return ret;
@@ -870,13 +848,16 @@ mymain(void)
 
     setenv("LIBVIRT_FAKE_ROOT_DIR", fakerootdir, 1);
 
-# define DETECT_MOUNTS(file) \
+# define DETECT_MOUNTS_FULL(file, fail) \
     do { \
+        struct _detectMountsData data = { file, fail }; \
         if (virTestRun("Detect cgroup mounts for " file, \
                        testCgroupDetectMounts, \
-                       file) < 0) \
+                       &data) < 0) \
             ret = -1; \
     } while (0)
+# define DETECT_MOUNTS(file) DETECT_MOUNTS_FULL(file, false);
+# define DETECT_MOUNTS_FAIL(file) DETECT_MOUNTS_FULL(file, true);
 
     DETECT_MOUNTS("ovirt-node-6.6");
     DETECT_MOUNTS("ovirt-node-7.1");
@@ -887,9 +868,10 @@ mymain(void)
     DETECT_MOUNTS("cgroups2");
     DETECT_MOUNTS("cgroups3");
     DETECT_MOUNTS("all-in-one");
-    DETECT_MOUNTS("no-cgroups");
+    DETECT_MOUNTS_FAIL("no-cgroups");
     DETECT_MOUNTS("kubevirt");
 
+    setenv("VIR_CGROUP_MOCK_FILENAME", "systemd", 1);
     if (virTestRun("New cgroup for self", testCgroupNewForSelf, NULL) < 0)
         ret = -1;
 
@@ -925,20 +907,21 @@ mymain(void)
 
     if (virTestRun("virCgroupGetPercpuStats works", testCgroupGetPercpuStats, NULL) < 0)
         ret = -1;
+    unsetenv("VIR_CGROUP_MOCK_FILENAME");
 
-    setenv("VIR_CGROUP_MOCK_MODE", "allinone", 1);
+    setenv("VIR_CGROUP_MOCK_FILENAME", "all-in-one", 1);
     if (virTestRun("New cgroup for self (allinone)", testCgroupNewForSelfAllInOne, NULL) < 0)
         ret = -1;
     if (virTestRun("Cgroup available", testCgroupAvailable, (void*)0x1) < 0)
         ret = -1;
-    unsetenv("VIR_CGROUP_MOCK_MODE");
+    unsetenv("VIR_CGROUP_MOCK_FILENAME");
 
-    setenv("VIR_CGROUP_MOCK_MODE", "logind", 1);
+    setenv("VIR_CGROUP_MOCK_FILENAME", "logind", 1);
     if (virTestRun("New cgroup for self (logind)", testCgroupNewForSelfLogind, NULL) < 0)
         ret = -1;
     if (virTestRun("Cgroup available", testCgroupAvailable, (void*)0x0) < 0)
         ret = -1;
-    unsetenv("VIR_CGROUP_MOCK_MODE");
+    unsetenv("VIR_CGROUP_MOCK_FILENAME");
 
     if (getenv("LIBVIRT_SKIP_CLEANUP") == NULL)
         virFileDeleteTree(fakerootdir);
