@@ -20,7 +20,6 @@
 
 #include <config.h>
 
-#include <stdlib.h>
 #include <signal.h>
 
 #include "testutils.h"
@@ -547,4 +546,4 @@ mymain(void)
     return ret == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
-VIRT_TEST_MAIN(mymain)
+VIR_TEST_MAIN(mymain)
