@@ -20,7 +20,6 @@
 
 #include <config.h>
 #include <sys/stat.h>
-#include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
 
@@ -741,4 +740,4 @@ mymain(void)
     return ret == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
-VIRT_TEST_MAIN(mymain)
+VIR_TEST_MAIN(mymain)
