@@ -28,17 +28,13 @@
 
 # include "virsh.h"
 
-virDomainPtr virshLookupDomainBy(vshControl *ctl,
-                                 const char *name,
-                                 unsigned int flags);
+struct virshDomainEventCallback {
+    const char *name;
+    virConnectDomainEventGenericCallback cb;
+};
+typedef struct virshDomainEventCallback virshDomainEventCallback;
 
-virDomainPtr virshCommandOptDomainBy(vshControl *ctl, const vshCmd *cmd,
-                                     const char **name, unsigned int flags);
-
-/* default is lookup by Id, Name and UUID */
-# define virshCommandOptDomain(_ctl, _cmd, _name)                      \
-    virshCommandOptDomainBy(_ctl, _cmd, _name,                         \
-                            VIRSH_BYID | VIRSH_BYUUID | VIRSH_BYNAME)
+extern virshDomainEventCallback virshDomainEventCallbacks[];
 
 extern const vshCmdDef domManagementCmds[];
 
