@@ -32,11 +32,19 @@ virNWFilterPtr
 virshCommandOptNWFilterBy(vshControl *ctl, const vshCmd *cmd,
                           const char **name, unsigned int flags);
 
+virNWFilterBindingPtr
+virshCommandOptNWFilterBindingBy(vshControl *ctl, const vshCmd *cmd,
+                                 const char **name, unsigned int flags);
+
 /* default is lookup by Name and UUID */
 # define virshCommandOptNWFilter(_ctl, _cmd, _name) \
     virshCommandOptNWFilterBy(_ctl, _cmd, _name, \
                               VIRSH_BYUUID | VIRSH_BYNAME)
 
+/* default is lookup by port dev */
+# define virshCommandOptNWFilterBinding(_ctl, _cmd, _name) \
+    virshCommandOptNWFilterBindingBy(_ctl, _cmd, _name, 0)
+
 extern const vshCmdDef nwfilterCmds[];
 
 #endif /* VIRSH_NWFILTER_H */
